@@ -15,18 +15,14 @@
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "../include/Utils.h"
-#include "../include/GParted_Core.h"
-#include "../include/PipeCapture.h"
+#include "Utils.h"
+#include "GParted_Core.h"
+#include "PipeCapture.h"
 
 #include <sstream>
 #include <fstream>
 #include <iomanip>
-#ifdef HAVE_GLIB_REGEX
 #include <glibmm/regex.h>
-#else
-#include <regex.h>
-#endif
 #include <locale.h>
 #include <uuid/uuid.h>
 #include <cerrno>
@@ -36,6 +32,8 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <string.h>
+#include <unistd.h>
 
 namespace GParted
 {
@@ -84,7 +82,7 @@ Glib::ustring Utils::num_to_str( Sector number )
 // Use palette from GNOME Human Interface Guidelines as a starting point.
 //     http://developer.gnome.org/hig-book/2.32/design-color.html.en
 //     http://web.archive.org/web/20130922173112/https://developer.gnome.org/hig-book/stable/design-color.html.en
-Glib::ustring Utils::get_color( FILESYSTEM filesystem )
+Glib::ustring Utils::get_color( FSType filesystem )
 {
 	switch( filesystem )
 	{
@@ -98,10 +96,11 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 		case FS_EXT3		: return "#7590AE" ;	//blue medium
 		case FS_EXT4		: return "#4B6983" ;	//blue dark
 		case FS_LINUX_SWAP	: return "#C1665A" ;	//red medium
+		case FS_LUKS		: return "#625B81" ;	//purple dark
 		case FS_F2FS		: return "#DF421E" ;	//accent red
 		case FS_FAT16		: return "#00FF00" ;	//green
 		case FS_FAT32		: return "#18D918" ;	// ~ medium green
-		case FS_EXFAT		: return "#2E8B57" ;	// ~ sea green
+		case FS_EXFAT		: return "#267726" ;	// Accent Green Dark
 		case FS_NILFS2		: return "#826647" ;	//face skin shadow
 		case FS_NTFS		: return "#42E5AC" ;	// ~ light aquamarine
 		case FS_REISERFS	: return "#ADA7C8" ;	//purple hilight
@@ -110,12 +109,14 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 		case FS_JFS			: return "#E0C39E" ;	//face skin medium
 		case FS_HFS			: return "#E0B6AF" ;	//red hilight
 		case FS_HFSPLUS		: return "#C0A39E" ;	// ~ serene red
+		case FS_UDF		: return "#105210";	// Accent Green Shadow
 		case FS_UFS			: return "#D1940C" ;	//accent yellow dark
 		case FS_USED		: return "#F8F8BA" ;	// ~ light tan yellow
 		case FS_UNUSED		: return "#FFFFFF" ;	//white
 		case FS_LVM2_PV		: return "#B39169" ;	//face skin dark
 		case FS_BITLOCKER	: return "#494066" ;	//purple shadow
-		case FS_LUKS		: return "#625B81" ;	//purple dark
+		case FS_GRUB2_CORE_IMG	: return "#666666" ;	//~ dark gray
+		case FS_ISO9660		: return "#D3D3D3" ;	// ~ light gray
 		case FS_LINUX_SWRAID   	: return "#5A4733" ;	// ~ dark brown
 		case FS_LINUX_SWSUSPEND	: return "#884631" ;	//red dark
 		case FS_REFS		: return "#2AB98A" ;	// ~ medium aquamarine
@@ -125,7 +126,7 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 	}
 }
 
-Glib::RefPtr<Gdk::Pixbuf> Utils::get_color_as_pixbuf( FILESYSTEM filesystem, int width, int height )
+Glib::RefPtr<Gdk::Pixbuf> Utils::get_color_as_pixbuf( FSType filesystem, int width, int height )
 {
 	Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create( Gdk::COLORSPACE_RGB, false, 8, width, height ) ;
 
@@ -184,7 +185,7 @@ int Utils::get_max_partition_name_length( Glib::ustring & tabletype )
 	return 0;
 }
 
-int Utils::get_filesystem_label_maxlength( FILESYSTEM filesystem )
+int Utils::get_filesystem_label_maxlength( FSType filesystem )
 {
 	switch( filesystem )
 	{
@@ -230,6 +231,7 @@ int Utils::get_filesystem_label_maxlength( FILESYSTEM filesystem )
 		case FS_NTFS		: return 128 ;
 		case FS_REISER4		: return 16 ;
 		case FS_REISERFS	: return 16 ;
+		case FS_UDF		: return 126;  // and only 63 if label contains character above U+FF
 		//case FS_UFS		: return  ;
 		case FS_XFS		: return 12 ;
 
@@ -238,7 +240,7 @@ int Utils::get_filesystem_label_maxlength( FILESYSTEM filesystem )
 }
 
 // Return libparted file system name / GParted display name
-Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
+Glib::ustring Utils::get_filesystem_string( FSType filesystem )
 {
 	switch( filesystem )
 	{
@@ -274,6 +276,7 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 		case FS_EXT3		: return "ext3" ;
 		case FS_EXT4		: return "ext4" ;
 		case FS_LINUX_SWAP	: return "linux-swap" ;
+		case FS_LUKS		: return "luks";
 		case FS_F2FS		: return "f2fs" ;
 		case FS_FAT16		: return "fat16" ;
 		case FS_FAT32		: return "fat32" ;
@@ -286,12 +289,14 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 		case FS_JFS		: return "jfs" ;
 		case FS_HFS		: return "hfs" ;
 		case FS_HFSPLUS		: return "hfs+" ;
+		case FS_UDF		: return "udf";
 		case FS_UFS		: return "ufs" ;
 		case FS_USED		: return _("used") ;
 		case FS_UNUSED		: return _("unused") ;
 		case FS_LVM2_PV		: return "lvm2 pv" ;
 		case FS_BITLOCKER	: return "bitlocker" ;
-		case FS_LUKS		: return "luks";
+		case FS_GRUB2_CORE_IMG	: return "grub2 core.img";
+		case FS_ISO9660		: return "iso9660";
 		case FS_LINUX_SWRAID	: return "linux-raid" ;
 		case FS_LINUX_SWSUSPEND	: return "linux-suspend" ;
 		case FS_REFS		: return "refs" ;
@@ -301,9 +306,23 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 	}
 }
 
+const Glib::ustring Utils::get_encrypted_string()
+{
+	/* TO TRANSLATORS: means that this is an encrypted file system */
+	return "[" + Glib::ustring( _("Encrypted") ) + "]";
+}
+
+const Glib::ustring Utils::get_filesystem_string( bool encrypted, FSType fstype )
+{
+	if ( encrypted )
+		return get_encrypted_string() + " " + get_filesystem_string( fstype );
+	else
+		return get_filesystem_string( fstype );
+}
+
 // Return Linux kernel name only for mountable file systems.
 // (Identical to a subset of the libparted names except that it's hfsplus instead of hfs+).
-const Glib::ustring Utils::get_filesystem_kernel_name( FILESYSTEM fstype )
+const Glib::ustring Utils::get_filesystem_kernel_name( FSType fstype )
 {
 	switch ( fstype )
 	{
@@ -322,13 +341,14 @@ const Glib::ustring Utils::get_filesystem_kernel_name( FILESYSTEM fstype )
 		case FS_NTFS     : return "ntfs";
 		case FS_REISER4  : return "reiser4";
 		case FS_REISERFS : return "reiserfs";
+		case FS_UDF      : return "udf";
 		case FS_UFS      : return "ufs";
 		case FS_XFS      : return "xfs";
 		default          : return "";
 	}
 }
 
-Glib::ustring Utils::get_filesystem_software( FILESYSTEM filesystem )
+Glib::ustring Utils::get_filesystem_software( FSType filesystem )
 {
 	switch( filesystem )
 	{
@@ -344,11 +364,12 @@ Glib::ustring Utils::get_filesystem_software( FILESYSTEM filesystem )
 		case FS_JFS         : return "jfsutils" ;
 		case FS_LINUX_SWAP  : return "util-linux" ;
 		case FS_LVM2_PV     : return "lvm2" ;
-		case FS_LUKS        : return "dmsetup";
+		case FS_LUKS        : return "cryptsetup, dmsetup";
 		case FS_NILFS2      : return "nilfs-utils" ;
 		case FS_NTFS        : return "ntfs-3g / ntfsprogs" ;
 		case FS_REISER4     : return "reiser4progs" ;
 		case FS_REISERFS    : return "reiserfsprogs / reiserfs-utils" ;
+		case FS_UDF         : return "udftools";
 		case FS_UFS         : return "" ;
 		case FS_XFS         : return "xfsprogs, xfsdump" ;
 
@@ -382,7 +403,7 @@ bool Utils::kernel_supports_fs( const Glib::ustring & fs )
 		return true ;
 
 	Glib::ustring output, error ;
-	execute_command( "modprobe " + fs, output, error, true );
+	execute_command( "modprobe " + Glib::shell_quote( fs ), output, error, true );
 
 	input .open( "/proc/filesystems" ) ;
 	if ( input )
@@ -496,7 +517,7 @@ double Utils::sector_to_unit( Sector sectors, Byte_Value sector_size, SIZE_UNIT
 int Utils::execute_command( const Glib::ustring & command )
 {
 	Glib::ustring dummy ;
-	return execute_command( command, dummy, dummy ) ;
+	return execute_command( command, NULL, dummy, dummy );
 }
 
 class CommandStatus
@@ -560,8 +581,18 @@ int Utils::execute_command( const Glib::ustring & command,
 			    Glib::ustring & output,
 			    Glib::ustring & error,
 			    bool use_C_locale )
+{
+	return execute_command( command, NULL, output, error, use_C_locale );
+}
+
+int Utils::execute_command( const Glib::ustring & command,
+                            const char * input,
+                            Glib::ustring & output,
+                            Glib::ustring & error,
+                            bool use_C_locale )
 {
 	Glib::Pid pid;
+	int in = -1;
 	// set up pipes for capture
 	int out, err;
 	CommandStatus status;
@@ -576,7 +607,7 @@ int Utils::execute_command( const Glib::ustring & command,
 			Glib::SPAWN_DO_NOT_REAP_CHILD | Glib::SPAWN_SEARCH_PATH,
 			use_C_locale ? sigc::ptr_fun( set_locale ) : sigc::slot< void >(),
 			&pid,
-			0,
+			( input != NULL ) ? &in : 0,
 			&out,
 			&err );
 	} catch (Glib::SpawnError &e) {
@@ -586,8 +617,6 @@ int Utils::execute_command( const Glib::ustring & command,
 	fcntl( out, F_SETFL, O_NONBLOCK );
 	fcntl( err, F_SETFL, O_NONBLOCK );
 	g_child_watch_add( pid, _store_exit_status, &status );
-	output.clear();
-	error.clear();
 	//Lock mutex so we have time to setup pipecapture for output and error streams
 	//  before connecting the input/output signal handler
 	if( !status.foreground )
@@ -599,6 +628,17 @@ int Utils::execute_command( const Glib::ustring & command,
 	outputcapture.connect_signal();
 	errorcapture.connect_signal();
 
+	if ( input != NULL && in != -1 )
+	{
+		// Write small amount of input to pipe to the child process.  Linux will
+		// always accept up 4096 bytes without blocking.  See pipe(7).
+		size_t len = strlen( input );
+		ssize_t written = write( in, input, len );
+		if ( written == -1 || (size_t)written < len )
+			std::cerr << "Write to child failed: " << Glib::strerror( errno ) << std::endl;
+		close( in );
+	}
+
 	if( status.foreground)
 		Gtk::Main::run();
 	else {
@@ -644,7 +684,6 @@ Glib::ustring Utils::regexp_label( const Glib::ustring & text
 {
 	//Extract text from a regular sub-expression or pattern.
 	//  E.g., "text we don't want (text we want)"
-#ifdef HAVE_GLIB_REGEX
 	std::vector<Glib::ustring> results;
 	Glib::RefPtr<Glib::Regex> myregexp =
 		Glib::Regex::create( pattern
@@ -657,20 +696,6 @@ Glib::ustring Utils::regexp_label( const Glib::ustring & text
 		return results[ 1 ] ;
 	else
 		return "" ;
-#else /* ! HAVE_GLIB_REGEX */
-	Glib::ustring label = "" ;
-	regex_t preg ;
-	int nmatch = 2 ;
-	regmatch_t pmatch[ 2 ] ;
-	int rc = regcomp( &preg, pattern .c_str(), REG_EXTENDED | REG_ICASE | REG_NEWLINE ) ;
-	if ( rc == 0 )
-	{
-		if ( regexec( &preg, text .c_str(), nmatch, pmatch, 0 ) == 0 )
-			label = text .substr( pmatch[ 1 ] .rm_so, pmatch[ 1 ] .rm_eo - pmatch[ 1 ] .rm_so ) ;
-		regfree( &preg ) ;
-	}
-	return label ;
-#endif
 }
 
 Glib::ustring Utils::trim( const Glib::ustring & src, const Glib::ustring & c /* = " \t\r\n" */ )
