@@ -15,10 +15,11 @@
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-
-#include "../include/btrfs.h"
-#include "../include/GParted_Core.h"
-#include "../include/Partition.h"
+#include "btrfs.h"
+#include "BlockSpecial.h"
+#include "FileSystem.h"
+#include "Mount_Info.h"
+#include "Partition.h"
 
 #include <ctype.h>
 
@@ -28,14 +29,15 @@ namespace GParted
 bool btrfs_found = false ;
 bool resize_to_same_size_fails = true ;
 
-//Cache of required btrfs file system device information by device
-//  E.g. For a single device btrfs on /dev/sda2 and a three device btrfs
-//       on /dev/sd[bcd]1 the cache would be:
-//  btrfs_device_cache["/dev/sda2"] = {devid=1, members=["/dev/sda2"]}
-//  btrfs_device_cache["/dev/sdb1"] = {devid=1, members=["/dev/sdd1", "/dev/sdc1", "/dev/sdb1"]}
-//  btrfs_device_cache["/dev/sdc1"] = {devid=2, members=["/dev/sdd1", "/dev/sdc1", "/dev/sdb1"]}
-//  btrfs_device_cache["/dev/sdd1"] = {devid=3, members=["/dev/sdd1", "/dev/sdc1", "/dev/sdb1"]}
-std::map<Glib::ustring, BTRFS_Device> btrfs_device_cache ;
+// Cache of required btrfs file system device information by device
+// E.g. For a single device btrfs on /dev/sda2 and a three device btrfs
+//      on /dev/sd[bcd]1 the cache would be as follows.  (Note that
+//      BS(str) is short hand for constructor BlockSpecial(str)).
+//  btrfs_device_cache[BS("/dev/sda2")] = {devid=1, members=[BS("/dev/sda2")]}
+//  btrfs_device_cache[BS("/dev/sdb1")] = {devid=1, members=[BS("/dev/sdd1"), BS("/dev/sdc1"), BS("/dev/sdb1")]}
+//  btrfs_device_cache[BS("/dev/sdc1")] = {devid=2, members=[BS("/dev/sdd1"), BS("/dev/sdc1"), BS("/dev/sdb1")]}
+//  btrfs_device_cache[BS("/dev/sdd1")] = {devid=3, members=[BS("/dev/sdd1"), BS("/dev/sdc1"), BS("/dev/sdb1")]}
+std::map<BlockSpecial, BTRFS_Device> btrfs_device_cache;
 
 FS btrfs::get_filesystem_support()
 {
@@ -110,7 +112,7 @@ FS btrfs::get_filesystem_support()
 	if ( ! Glib::find_program_in_path( "btrfstune" ).empty() )
 	{
 		Utils::execute_command( "btrfstune --help", output, error, true );
-		if ( Utils::regexp_label( error, "^[[:blank:]]*(-u)[[:blank:]]" ) == "-u" )
+		if ( Utils::regexp_label( output + error, "^[[:blank:]]*(-u)[[:blank:]]" ) == "-u" )
 			fs.write_uuid = FS::EXTERNAL;
 	}
 
@@ -129,7 +131,7 @@ FS btrfs::get_filesystem_support()
 	}
 #endif
 
-	fs .MIN = 256 * MEBIBYTE ;
+	fs_limits.min_size = 256 * MEBIBYTE;
 
 	//Linux before version 3.2 fails when resizing btrfs file system
 	//  to the same size.
@@ -153,14 +155,15 @@ bool btrfs::is_busy( const Glib::ustring & path )
 
 bool btrfs::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "mkfs.btrfs -L \"" + new_partition.get_filesystem_label() + "\" " +
-	                          new_partition.get_path(),
+	return ! execute_command( "mkfs.btrfs -L " + Glib::shell_quote( new_partition.get_filesystem_label() ) +
+	                          " " + Glib::shell_quote( new_partition.get_path() ),
 	                          operationdetail, EXEC_CHECK_STATUS );
 }
 
 bool btrfs::check_repair( const Partition & partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "btrfsck " + partition .get_path(), operationdetail, EXEC_CHECK_STATUS );
+	return ! execute_command( "btrfsck " + Glib::shell_quote( partition.get_path() ),
+	                          operationdetail, EXEC_CHECK_STATUS );
 }
 
 void btrfs::set_used_sectors( Partition & partition )
@@ -216,9 +219,11 @@ void btrfs::set_used_sectors( Partition & partition )
 	//  4) Extents can be and are relocated to other devices within the file system
 	//     when shrinking a device.
 	if ( btrfs_found )
-		Utils::execute_command( "btrfs filesystem show " + partition .get_path(), output, error, true ) ;
+		Utils::execute_command( "btrfs filesystem show " + Glib::shell_quote( partition.get_path() ),
+		                        output, error, true );
 	else
-		Utils::execute_command( "btrfs-show " + partition .get_path(), output, error, true ) ;
+		Utils::execute_command( "btrfs-show " + Glib::shell_quote( partition.get_path() ),
+		                        output, error, true );
 	//In many cases the exit status doesn't reflect valid output or an error condition
 	//  so rely on parsing the output to determine success.
 
@@ -294,8 +299,8 @@ void btrfs::set_used_sectors( Partition & partition )
 
 bool btrfs::write_label( const Partition & partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "btrfs filesystem label " + partition.get_path() +
-	                          " \"" + partition.get_filesystem_label() + "\"",
+	return ! execute_command( "btrfs filesystem label " + Glib::shell_quote( partition.get_path() ) +
+	                          " " + Glib::shell_quote( partition.get_filesystem_label() ),
 	                          operationdetail, EXEC_CHECK_STATUS );
 }
 
@@ -319,7 +324,8 @@ bool btrfs::resize( const Partition & partition_new, OperationDetail & operation
 		mount_point = mk_temp_dir( "", operationdetail ) ;
 		if ( mount_point .empty() )
 			return false ;
-		success &= ! execute_command( "mount -v -t btrfs " + path + " " + mount_point,
+		success &= ! execute_command( "mount -v -t btrfs " + Glib::shell_quote( path ) +
+		                              " " + Glib::shell_quote( mount_point ),
 		                              operationdetail, EXEC_CHECK_STATUS );
 	}
 	else
@@ -335,9 +341,9 @@ bool btrfs::resize( const Partition & partition_new, OperationDetail & operation
 			size = "max" ;
 		Glib::ustring cmd ;
 		if ( btrfs_found )
-			cmd = "btrfs filesystem resize " + devid_str + ":" + size + " " + mount_point ;
+			cmd = "btrfs filesystem resize " + devid_str + ":" + size + " " + Glib::shell_quote( mount_point );
 		else
-			cmd = "btrfsctl -r " + devid_str + ":" + size + " " + mount_point ;
+			cmd = "btrfsctl -r " + devid_str + ":" + size + " " + Glib::shell_quote( mount_point );
 		exit_status = execute_command( cmd, operationdetail );
 		bool resize_succeeded = ( exit_status == 0 ) ;
 		if ( resize_to_same_size_fails )
@@ -364,8 +370,8 @@ bool btrfs::resize( const Partition & partition_new, OperationDetail & operation
 		success &= resize_succeeded ;
 
 		if ( ! partition_new .busy )
-			success &= ! execute_command( "umount -v " + mount_point, operationdetail,
-			                              EXEC_CHECK_STATUS );
+			success &= ! execute_command( "umount -v " + Glib::shell_quote( mount_point ),
+			                              operationdetail, EXEC_CHECK_STATUS );
 	}
 
 	if ( ! partition_new .busy )
@@ -377,9 +383,11 @@ bool btrfs::resize( const Partition & partition_new, OperationDetail & operation
 void btrfs::read_label( Partition & partition )
 {
 	if ( btrfs_found )
-		Utils::execute_command( "btrfs filesystem show " + partition .get_path(), output, error, true ) ;
+		Utils::execute_command( "btrfs filesystem show " + Glib::shell_quote( partition.get_path() ),
+		                        output, error, true );
 	else
-		Utils::execute_command( "btrfs-show " + partition .get_path(), output, error, true ) ;
+		Utils::execute_command( "btrfs-show " + Glib::shell_quote( partition.get_path() ),
+		                        output, error, true );
 	//In many cases the exit status doesn't reflect valid output or an error condition
 	//  so rely on parsing the output to determine success.
 
@@ -413,9 +421,11 @@ void btrfs::read_label( Partition & partition )
 void btrfs::read_uuid( Partition & partition )
 {
 	if ( btrfs_found )
-		Utils::execute_command( "btrfs filesystem show " + partition .get_path(), output, error, true ) ;
+		Utils::execute_command( "btrfs filesystem show " + Glib::shell_quote( partition.get_path() ),
+		                        output, error, true );
 	else
-		Utils::execute_command( "btrfs-show " + partition .get_path(), output, error, true ) ;
+		Utils::execute_command( "btrfs-show " + Glib::shell_quote( partition.get_path() ),
+		                        output, error, true );
 	//In many cases the exit status doesn't reflect valid output or an error condition
 	//  so rely on parsing the output to determine success.
 
@@ -434,7 +444,8 @@ void btrfs::read_uuid( Partition & partition )
 
 bool btrfs::write_uuid( const Partition & partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "btrfstune -f -u " + partition.get_path(), operationdetail, EXEC_CHECK_STATUS );
+	return ! execute_command( "btrfstune -f -u " + Glib::shell_quote( partition.get_path() ),
+	                          operationdetail, EXEC_CHECK_STATUS );
 }
 
 void btrfs::clear_cache()
@@ -454,21 +465,24 @@ Glib::ustring btrfs::get_mount_device( const Glib::ustring & path )
 		//  Use fallback busy detection method which can only determine if the
 		//  mounting device is mounted or not, not any of the other members of a
 		//  multi-device btrfs file system.
-		if ( GParted_Core::is_dev_mounted( path ) )
+		if ( Mount_Info::is_dev_mounted( path ) )
 			return path ;
 		return "" ;
 	}
 
 	for ( unsigned int i = 0 ; i < btrfs_dev .members .size() ; i ++ )
-		if ( GParted_Core::is_dev_mounted( btrfs_dev .members[ i ] ) )
-			return btrfs_dev .members[ i ] ;
+		if ( Mount_Info::is_dev_mounted( btrfs_dev.members[i] ) )
+			return btrfs_dev.members[i].m_name;
 	return "" ;
 }
 
 std::vector<Glib::ustring> btrfs::get_members( const Glib::ustring & path )
 {
 	BTRFS_Device btrfs_dev = get_cache_entry( path ) ;
-	return btrfs_dev .members ;
+	std::vector<Glib::ustring> membs;
+	for ( unsigned int i = 0 ; i < btrfs_dev.members.size() ; i ++ )
+		membs.push_back( btrfs_dev.members[i].m_name );
+	return membs;
 }
 
 //Private methods
@@ -476,7 +490,7 @@ std::vector<Glib::ustring> btrfs::get_members( const Glib::ustring & path )
 //Return btrfs device cache entry, incrementally loading cache as required
 const BTRFS_Device & btrfs::get_cache_entry( const Glib::ustring & path )
 {
-	std::map<Glib::ustring, BTRFS_Device>::const_iterator bd_iter = btrfs_device_cache .find( path ) ;
+	std::map<BlockSpecial, BTRFS_Device>::const_iterator bd_iter = btrfs_device_cache.find( BlockSpecial( path ) );
 	if ( bd_iter != btrfs_device_cache .end() )
 		return bd_iter ->second ;
 
@@ -484,9 +498,9 @@ const BTRFS_Device & btrfs::get_cache_entry( const Glib::ustring & path )
 	std::vector<int> devid_list ;
 	std::vector<Glib::ustring> path_list ;
 	if ( btrfs_found )
-		Utils::execute_command( "btrfs filesystem show " + path, output, error, true ) ;
+		Utils::execute_command( "btrfs filesystem show " + Glib::shell_quote( path ), output, error, true );
 	else
-		Utils::execute_command( "btrfs-show " + path, output, error, true ) ;
+		Utils::execute_command( "btrfs-show " + Glib::shell_quote( path ), output, error, true );
 	//In many cases the exit status doesn't reflect valid output or an error condition
 	//  so rely on parsing the output to determine success.
 
@@ -511,15 +525,18 @@ const BTRFS_Device & btrfs::get_cache_entry( const Glib::ustring & path )
 		offset = index + 5 ;  //Next find starts immediately after current "devid"
 	}
 	//Add cache entries for all found devices
+	std::vector<BlockSpecial> bs_list;
+	for ( unsigned int i = 0 ; i < path_list.size() ; i ++ )
+		bs_list.push_back( BlockSpecial( path_list[i] ) );
 	for ( unsigned int i = 0 ; i < devid_list .size() ; i ++ )
 	{
 		BTRFS_Device btrfs_dev ;
 		btrfs_dev .devid = devid_list[ i ] ;
-		btrfs_dev .members = path_list ;
-		btrfs_device_cache[ path_list[ i ] ] = btrfs_dev ;
+		btrfs_dev.members = bs_list;
+		btrfs_device_cache[ BlockSpecial( path_list[i] ) ] = btrfs_dev;
 	}
 
-	bd_iter = btrfs_device_cache .find( path ) ;
+	bd_iter = btrfs_device_cache.find( BlockSpecial( path ) );
 	if ( bd_iter != btrfs_device_cache .end() )
 		return bd_iter ->second ;
 
