@@ -14,10 +14,10 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
- 
- 
-#include "../include/hfsplus.h"
-#include "../include/Partition.h"
+
+#include "hfsplus.h"
+#include "FileSystem.h"
+#include "Partition.h"
 
 namespace GParted
 {
@@ -42,9 +42,6 @@ FS hfsplus::get_filesystem_support()
 	if ( ! Glib::find_program_in_path( "fsck.hfsplus" ) .empty() )
 		fs .check = FS::EXTERNAL ;
 
-	if ( ! Glib::find_program_in_path( "vol_id" ) .empty() )
-		fs .read_label = FS::EXTERNAL ;
-
 	fs .copy = GParted::FS::GPARTED ;
 	fs .move = GParted::FS::GPARTED ;
 	fs .online_read = FS::GPARTED ;
@@ -52,40 +49,21 @@ FS hfsplus::get_filesystem_support()
 	return fs ;
 }
 
-void hfsplus::read_label( Partition & partition )
-{
-	if ( ! Utils::execute_command( "vol_id " + partition .get_path(), output, error, true ) )
-	{
-		Glib::ustring label = Utils::regexp_label( output, "ID_FS_LABEL=([^\n]*)" ) ;
-		//FIXME: find a better way to see if label is empty.. imagine someone uses 'untitled' as label.... ;)
-		if ( label != "untitled" )
-			partition.set_filesystem_label( label );
-		else
-			partition.set_filesystem_label( "" );
-	}
-	else
-	{
-		if ( ! output .empty() )
-			partition.push_back_message( output );
-		
-		if ( ! error .empty() )
-			partition.push_back_message( error );
-	}
-}
-
 bool hfsplus::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
 	Glib::ustring cmd = "";
 	if( new_partition.get_filesystem_label().empty() )
-		cmd = "mkfs.hfsplus " + new_partition .get_path() ;
+		cmd = "mkfs.hfsplus " + Glib::shell_quote( new_partition.get_path() );
 	else
-		cmd = "mkfs.hfsplus -v \"" + new_partition.get_filesystem_label() + "\" " + new_partition.get_path();
+		cmd = "mkfs.hfsplus -v " + Glib::shell_quote( new_partition.get_filesystem_label() ) +
+		      " " + Glib::shell_quote( new_partition.get_path() );
 	return ! execute_command( cmd , operationdetail, EXEC_CHECK_STATUS );
 }
 
 bool hfsplus::check_repair( const Partition & partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "fsck.hfsplus -f -y " + partition.get_path(), operationdetail, EXEC_CHECK_STATUS );
+	return ! execute_command( "fsck.hfsplus -f -y " + Glib::shell_quote( partition.get_path() ),
+	                          operationdetail, EXEC_CHECK_STATUS );
 }
 
 } //GParted
